//---------------------------------------------------------------------------
#include "sros.h"
#include "OSCore.h"
#include "device2.h"
#include "3052.h"
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  LN^[foCXhCo֐ `
//---------------------------------------------------------------------------
/***************************************************

	LN^[foCXhCo֐

 ***************************************************/

#define BR_9600BPS 80
#define BR14400BPS 59
#define BR19200BPS 39
#define BR38400BPS 19
#define BR57600BPS 12

#define SIO SCI1

static int DevNo;

//---------------------------------------------------------------------------
//  [U[C^[tF[X
//---------------------------------------------------------------------------
char GetSCI(void)
{
	return GetDEV(DevNo);
}

void PutSCI(char c)
{
	PutDEV(DevNo, c);
}

BOOL EmptyReadBufSCI(void)
{
	return EmptyReadBuf(DevNo);
}

BOOL EmptyWriteBufSCI(void)
{
	return EmptyWriteBuf(DevNo);
}

void PrintSCI(char * msg)
{
	PrintDEV(DevNo, msg);
}

void ControlSCI(char * msg)
{
	int i;


#ifndef TCPP
	SIO.SCR.BYTE = 0x00;

	if     (StrPos(msg,  "9600") > -1)	SIO.BRR = BR_9600BPS;
	else if(StrPos(msg, "14400") > -1)  SIO.BRR = BR14400BPS;
	else if(StrPos(msg, "19200") > -1)  SIO.BRR = BR19200BPS;
	else if(StrPos(msg, "38400") > -1)  SIO.BRR = BR38400BPS;
	else if(StrPos(msg, "57600") > -1)  SIO.BRR = BR57600BPS;

	if(     StrPos(msg, "7bit" ) > -1)	SIO.SMR.BIT.CHR = 1;
	else if(StrPos(msg, "8bit" ) > -1)	SIO.SMR.BIT.CHR = 0;

	if(     StrPos(msg, "Stop1") > -1)	SIO.SMR.BIT.STOP = 0;
	else if(StrPos(msg, "Stop2") > -1)	SIO.SMR.BIT.STOP = 1;

	if(     StrPos(msg, "Sync" ) > -1)	SIO.SMR.BIT.CA = 1;
	else if(StrPos(msg, "Async") > -1)	SIO.SMR.BIT.CA = 0;

	if(     StrPos(msg, "NONE" ) > -1)	SIO.SMR.BIT.PE = 0;
	else
	{
										SIO.SMR.BIT.PE = 1;
		if     (StrPos(msg, "ODD") > -1)SIO.SMR.BIT.OE = 1;
		else if(StrPos(msg, "EVN") > -1)SIO.SMR.BIT.OE = 0;
	}
	for(i=0;i<280;i++)	{}

	SIO.SCR.BYTE = 0x70;

	i = SIO.SSR.BYTE;
	SIO.SSR.BYTE = 0x80;

#endif
}

//---------------------------------------------------------------------------
//  hCoC^[tF[X
//---------------------------------------------------------------------------
static BOOL RD_Ready(void)
{
#ifndef TCPP
	return SIO.SSR.BIT.RDRF;
#endif
}

static char IRQ_Read(void)
{
	unsigned char  i, c;

#ifndef TCPP
	c = SIO.RDR;
	SIO.SSR.BYTE &= 0xbf;
#endif
	return c;
}

static BOOL WR_Ready(void)
{
#ifndef TCPP
	return SIO.SSR.BIT.TDRE;
#endif
}

static void IRQ_Write(char c)
{
	unsigned char i;

#ifndef TCPP
	SIO.TDR = c;
	i = SIO.SSR.BYTE;
	SIO.SSR.BIT.TDRE	= 0;
	SIO.SCR.BIT.TIE	= 1;
#endif
}

#ifndef TCPP
void int_eri1()  __attribute__ ((interrupt_handler));
void int_rxi1()  __attribute__ ((interrupt_handler));
void int_txi1()  __attribute__ ((interrupt_handler));
void int_tei1()  __attribute__ ((interrupt_handler));
#endif

void int_eri1 (void)
{
	int i;

#ifndef TCPP
	i = SIO.SSR.BYTE;
	SIO.SSR.BYTE = 0;
#endif
}

void int_rxi1 (void)
{
	IRQ_ReadDEV(DevNo);
}

void int_txi1 (void)
{
	IRQ_WriteDEV(DevNo);
}

void int_tei1 (void)
{
#ifndef TCPP
	SIO.SSR.BIT.TDRE = 1;
#endif
}

//---------------------------------------------------------------------------
//  [`
//---------------------------------------------------------------------------
static void InitDEV(void)
{
	int i;

#ifndef TCPP
	SIO.SMR.BYTE = 0x00;
#endif
	ControlSCI("Async 19200 8bit Stop1 NONE ");
}

static void Free(void)
{
#ifndef TCPP
	SIO.SCR.BYTE = 0x00;
	SIO.SMR.BYTE = 0x00;
#endif
	SetDevice(DevNo, 0);
}

void AddSCI(int No)
{
	static DEVICE dev;

	DevNo 				= No;
	dev.Init			= InitDEV;
	dev.Ctrl			= ControlSCI;
	dev.IRQ_ReadDevice 	= IRQ_Read;
	dev.RD_Ready		= RD_Ready;
	dev.IRQ_WriteDevice = IRQ_Write;
	dev.WR_Ready		= WR_Ready;
	dev.Free			= Free;

	SetDevice(DevNo, &dev);
}
