//---------------------------------------------------------------------------
#include "SROS.h"
#include "OSCore.h"
#include "BIOS.h"
#include "Device1.h"
#include "Device2.h"
#include "Device3.h"
#include "Device4.h"

//---------------------------------------------------------------------------

#define SCI_DEV DEV1
#define LCD_DEV DEV2

void RegistDriver(void)
{
	AddSCI(SCI_DEV);
	AddLCD(LCD_DEV);
//	AddDEV3(DEV3);
//	AddDEV4(DEV4);
}

void init_io(void)
{
	IO.PMR1.BYTE 	= 0x02;
	IO.PCR1 		= 0xF7;
	IO.PCR2 		= 0x06;

	IO.PMR5.BYTE	= 0x03;
	IO.PCR5			= 0x30;
	IO.PUCR5.BYTE	= 0x0F;

	IO.PCR7			= 0x70;

	IO.PCR8			= 0x0F;
}

void init_systimer(void)
{
	TV.TCSRV.BYTE = 0x00;
	TV.TCRV0.BYTE = 0xCA;
	TV.TCORA 	  = 250;
	TV.TCORB	  = 125;
	IENR1.BIT.IENTA = 0;
}

void init_adc(void);

void init_bios(void)
{
	init_adc();
}

/*****************************************************

	VXehn|[g̃C^[tF[X

 *****************************************************/
void PoleIO(void)
{
	PolingDEV(LCD_DEV);
}

/*==========================================================================
							ADC hCo
==========================================================================*/
struct tag_ADCData ADC_Data;

#define ADCSR AD.ADCSR

static void InitADData(int index, WORD data)
{
	int i;

	ADC_Data.data[index] = data;
	ADC_Data.SUM[index] = 0;
	for (i = 0; i < 8; i++)
	{
		ADC_Data.AVGDATA[index][i]= data;
		ADC_Data.SUM[index] = ADC_Data.SUM[index] + data;
	}
}

static void StartADC(int Select)
{
	int i;

	if( Select < 4) ADCSR.BIT.SCAN = 0;
	else            ADCSR.BIT.SCAN = 1;
	ADCSR.BIT.CH = Select;
	ADCSR.BYTE = ADCSR.BYTE | 0x20;
	ADC_Data.Select = Select;
}

void SetMoveAvg(int Select, WORD data)
{
	int temp;

	ADC_Data.SUM[Select] = ADC_Data.SUM[Select]
						 - ADC_Data.AVGDATA[Select][ADC_Data.Count]
						 + data;
	ADC_Data.AVGDATA[Select][ADC_Data.Count] = data;
	temp = ADC_Data.SUM[Select];
	temp = temp >> 3;
	ADC_Data.data[Select] = temp;
}

void init_adc(void)
{
	int i;

	ADCSR.BIT.ADST = 0;
	ADCSR.BYTE    = ADCSR.BYTE | 0x58;
	for(i = 0; i < 8; i++)
	{
		InitADData(i, 0);
	}
}

void int_adi()  __attribute__ ((interrupt_handler));
void int_adi(void)
{
	ADCSR.BYTE    = (ADCSR.BYTE & 0x58) | 0x47;
	switch(ADC_Data.Select){
	case 0:
		PostMessage(GetDefTask(), MSG_AD1FINE, AD.ADDRA);
		break;

	case 1:
		PostMessage(GetDefTask(), MSG_AD2FINE, AD.ADDRB);
		break;

	case 2:
		PostMessage(GetDefTask(), MSG_AD3FINE, AD.ADDRC);
		break;

	case 3:
		PostMessage(GetDefTask(), MSG_AD4FINE, AD.ADDRD);
		break;

	case 4:
		PostMessage(GetDefTask(), MSG_AD5FINE, AD.ADDRA);
		break;

	case 5:
		PostMessage(GetDefTask(), MSG_AD6FINE, AD.ADDRB);
		break;

	case 6:
		PostMessage(GetDefTask(), MSG_AD7FINE, AD.ADDRC);
		break;

	case 7:
		PostMessage(GetDefTask(), MSG_AD8FINE, AD.ADDRD);
		break;
	}
}

/*==========================================================================
							LED hCo
==========================================================================*/
void WriteLEDPort(BYTE value)
{
	BYTE temp;

	temp = IO.PDR8.BYTE & 0x0F;
	IO.PDR8.BYTE = temp | (~value & 0xF0);
}

/*==========================================================================
							rv hCo
==========================================================================*/
BYTE GetSW(void)
{
	return (~IO.PDR8.BYTE >> 4) & SW_MASK;
}

BYTE GetDIPSW(void)
{
	return 0;
}

//	XCb`Cxg̊g

void SWEvent(BYTE data)
{
	SW_Ancer( 0, SW1_MASK, data, MSG_SW1ON, MSG_SW1OFF);
	SW_Ancer( 1, SW2_MASK, data, MSG_SW2ON, MSG_SW2OFF);
	SW_Ancer( 2, SW3_MASK, data, MSG_SW3ON, MSG_SW3OFF);
	SW_Ancer( 3, SW4_MASK, data, MSG_SW4ON, MSG_SW4OFF);
//	SW_Ancer( 4, SW5_MASK, data, MSG_SW5ON, MSG_SW5OFF);
//	SW_Ancer( 5, SW6_MASK, data, MSG_SW6ON, MSG_SW6OFF);
//	SW_Ancer( 6, SW7_MASK, data, MSG_SW7ON, MSG_SW7OFF);
//	SW_Ancer( 7, SW8_MASK, data, MSG_SW8ON, MSG_SW8OFF);
}

/*****************************************************

	荞ݏ

 *****************************************************/
/*==========================================================================
							^C}[荞
==========================================================================*/
void int_timerv()  __attribute__ ((interrupt_handler));

static long sp;
extern THRDDATA THRD;

#define STACK_SIZE 256
static struct {
	int mem[STACK_SIZE];
} stack;

long ThreadStckTop(int Select)
{
	switch(Select){
	case 1:
		return (long) stack.mem+STACK_SIZE;
	case 2:
		return (long) stack.mem+STACK_SIZE-128;
	case 3:
		return (long) stack.mem+STACK_SIZE-192;
	}
}

void reset_TTimer(void)
{
	TV.TCORB = TV.TCNTV-1;
	if(TV.TCORB > TV.TCORA)
	{
		TV.TCORB = TV.TCORB % TV.TCORA;
	}
}

void int_timerv (void)
{
	static int count=0;

	if((TV.TCSRV.BIT.CMFA & TV.TCRV0.BIT.CMIEA) == 1)
	{
		switch(++count){
		case  50:
			Timer100mSec();
			count = 0;
		}
		if(count % 5 == 0)
			Timer10mSec();
		SystemTimer();

		TV.TCSRV.BIT.CMFA = 0;
	}
	if((TV.TCSRV.BIT.CMFB & TV.TCRV0.BIT.CMIEB) == 1)
	{
__asm__(
"	push.l	er1\n"
"	push.l	er2\n"
);
	if(THRD.Count > 1)
	{
__asm__(
"	push.l	er3\n"
"	push.l	er4\n"
"	push.l	er5\n"
"	push.l	er6\n"
"	mov.l	er7,@_sp\n"
);
		reset_TTimer();
		ChangeThread(&sp);
__asm__(
"	mov.l	@_sp,er7\n"
"	pop.l	er6\n"
"	pop.l	er5\n"
"	pop.l	er4\n"
"	pop.l	er3\n"
);
	}
__asm__(
"	pop.l	er2\n"
"	pop.l	er1\n"
);
		TV.TCSRV.BIT.CMFB = 0;
	}
	if((TV.TCSRV.BIT.OVF & TV.TCRV0.BIT.OVIE) == 1)
	{

		TV.TCSRV.BIT.OVF = 0;
	}
}

/*==========================================================================
							gbv
==========================================================================*/
void int_trap0()  __attribute__ ((interrupt_handler));
void int_trap0(void)
{
__asm__(
"	push.l	er1\n"
"	push.l	er2\n"
);
	if(THRD.Count > 1)
	{
__asm__(
"	push.l	er3\n"
"	push.l	er4\n"
"	push.l	er5\n"
"	push.l	er6\n"
"	mov.l	er7,@_sp\n"
);
		reset_TTimer();
		ChangeThread(&sp);
__asm__(
"	mov.l	@_sp,er7\n"
"	pop.l	er6\n"
"	pop.l	er5\n"
"	pop.l	er4\n"
"	pop.l	er3\n"
);
	}
__asm__(
"	pop.l	er2\n"
"	pop.l	er1\n"
);
}

