//---------------------------------------------------------------------------
#include "sros.h"
#include "OSCore.h"
#include "device2.h"
#include "3664.h"
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  LN^[foCXhCo֐ `
//---------------------------------------------------------------------------
/***************************************************

	LN^[foCXhCo֐

 ***************************************************/

#define BR_9600BPS 51
#define BR14400BPS 38
#define BR19200BPS 25
#define BR38400BPS 12
#define BR57600BPS  8

#define SIO SCI3

static int DevNo;

//---------------------------------------------------------------------------
//  [U[C^[tF[X
//---------------------------------------------------------------------------
char GetSCI(void)
{
	return GetDEV(DevNo);
}

void PutSCI(char c)
{
	PutDEV(DevNo, c);
}

BOOL EmptyReadBufSCI(void)
{
	return EmptyReadBuf(DevNo);
}

BOOL EmptyWriteBufSCI(void)
{
	return EmptyWriteBuf(DevNo);
}

void PrintSCI(char * msg)
{
	PrintDEV(DevNo, msg);
}

void ControlSCI(char * msg)
{
	int i;
	BYTE DATA;

#ifndef TCPP
	i = SIO.SSR.BYTE;
	SIO.SSR.BYTE = 0x80;

	if     (StrPos(msg,  "9600") > -1)	SIO.BRR = BR_9600BPS;
	else if(StrPos(msg, "14400") > -1)  SIO.BRR = BR14400BPS;
	else if(StrPos(msg, "19200") > -1)  SIO.BRR = BR19200BPS;
	else if(StrPos(msg, "38400") > -1)  SIO.BRR = BR38400BPS;
	else if(StrPos(msg, "57600") > -1)  SIO.BRR = BR57600BPS;

	for(i=0;i<280;i++)	{}

	DATA = SIO.SMR.BYTE;
	if(     StrPos(msg, "7bit" ) > -1)	DATA |= 0x02;
	else if(StrPos(msg, "8bit" ) > -1)	DATA &= 0x0D;

	if(     StrPos(msg, "Stop2") > -1)	DATA |= 0x10;
	else if(StrPos(msg, "Stop1") > -1)	DATA &= 0xEF;

	if(     StrPos(msg, "Sync" ) > -1)	DATA |= 0x01;
	else if(StrPos(msg, "Async") > -1)	DATA &= 0xFE;

	if(     StrPos(msg, "NONE" ) > -1)	DATA &= 0xFB;
	else
	{
										DATA |= 0x04;
		if     (StrPos(msg, "ODD") > -1)DATA |= 0x08;
		else if(StrPos(msg, "EVN") > -1)DATA &= 0xF7;
	}

	SIO.SMR.BYTE = DATA;

	SCR.BYTE = 0x70;

	i = SIO.SSR.BYTE;
	SIO.SSR.BYTE = 0x80;
	for(i=0;i<280;i++)	{}
	PutSCI(0x0A);

#endif
}

//---------------------------------------------------------------------------
//  hCoC^[tF[X
//---------------------------------------------------------------------------
static BOOL RD_Ready(void)
{
#ifndef TCPP
	return SIO.SSR.BIT.RDRF;
#endif
}

static char IRQ_Read(void)
{
	unsigned char  i, c;

#ifndef TCPP
	c = SIO.RDR;
	SIO.SSR.BYTE &= 0xbf;
#endif
	return c;
}

static BOOL WR_Ready(void)
{
#ifndef TCPP
	return SIO.SSR.BIT.TDRE;
#endif
}

static void IRQ_Write(char c)
{
	unsigned char i;

#ifndef TCPP
	SIO.TDR = c;
	i = SIO.SSR.BYTE;
	SIO.SSR.BIT.TDRE	= 0;
	SIO.SCR3.BIT.TIE	= 1;
#endif
}

#ifndef TCPP
void int_sci3()  __attribute__ ((interrupt_handler));
#endif

void int_sci3 (void)
{
	if(SCI3.SCR3.BIT.TEIE & SIO.SSR.BIT.TEND)
	{
		SIO.SSR.BIT.TEND = 0;
	}
	if(SCI3.SCR3.BIT.TIE & SIO.SSR.BIT.TDRE)
	{
		IRQ_WriteDEV(DevNo);
	}
	if(SCI3.SCR3.BIT.RIE & SIO.SSR.BIT.RDRF)
	{
		IRQ_ReadDEV(DevNo);
	}

}

//---------------------------------------------------------------------------
//  [`
//---------------------------------------------------------------------------
static void InitDEV(void)
{
	int i;

#ifndef TCPP
	SIO.SMR.BYTE = 0x00;
#endif
	ControlSCI("Async 19200 8bit Stop1 NONE ");
}

static void Free(void)
{
#ifndef TCPP
	SIO.SCR3.BYTE = 0x00;
	SIO.SMR.BYTE = 0x00;
#endif
	SetDevice(DevNo, 0);
}

void AddSCI(int No)
{
	static DEVICE dev;
	int i;

	DevNo 				= No;
	dev.Init			= InitDEV;
	dev.Ctrl			= ControlSCI;
	dev.IRQ_ReadDevice 	= IRQ_Read;
	dev.RD_Ready		= RD_Ready;
	dev.IRQ_WriteDevice = IRQ_Write;
	dev.WR_Ready		= WR_Ready;
	dev.Free			= Free;

	dev.Buffer[0].count = 0;
	dev.Buffer[1].count = 0;
	dev.Buffer[0].top   = 0;
	dev.Buffer[1].top   = 0;

	for(i = 0; i < MAX_BUFFER; i++)
	{
		dev.Buffer[0].buf[i] = 0;
		dev.Buffer[1].buf[i] = 0;
	}
	SetDevice(DevNo, &dev);
}
