//---------------------------------------------------------------------------
#include "SROS.h"
#include "OSCore.h"
#include "BIOS.h"
#include "Shell.h"

//---------------------------------------------------------------------------

/*******************************************************

	^XNAPI

 *******************************************************/

struct _tag_tcb;

typedef struct _tag_tcb {
	BYTE Flag;
	MSG msg;
	MSGPROC * MsgProc;
	int IN, OUT;
	struct _tag_tcb * Next;
} TCBD;

TCBD TCB[MAX_TCB];
TCBD *root;
TCBD *troot;
MSG * DefCon;

static void init_task(void)
{
	int i;

	root = NULL;
	troot = NULL;

	for(i = 0; i < MAX_TCB; i++)
	{
		TCB[i].Flag = 0;
		TCB[i].msg.top   = 0;
		TCB[i].msg.count = 0;
		TCB[i].MsgProc   = NULL;
		TCB[i].Next      = NULL;
	}
}

MSG * MakeTask(MSGPROC * proc, BYTE Flag)
{
	int i;
	TCBD * tcb;

	for(i = 0; i < MAX_TCB; i++)
	{
		if(TCB[i].Flag == MODE_NONE)
		{
			TCB[i].MsgProc = proc;
			TCB[i].Flag    = Flag;

			if(Flag == MODE_TIMER)
			{
				if(troot == NULL)
				{
					troot = &TCB[i];
					PostMessage(&troot->msg, MSG_INIT, 0);
					return &TCB[i].msg;
				}
				tcb = troot;
			}
			else
			{
				if(root == NULL)
				{
					root = &TCB[i];
					PostMessage(&root->msg, MSG_INIT, 0);
					return &TCB[i].msg;
				}
				tcb = root;
			}
			while(tcb->Next) tcb = tcb->Next;
			tcb->Next = &TCB[i];
			PostMessage(&TCB[i].msg, MSG_INIT, 0);
			return &TCB[i].msg;
		}
	}
	return NULL;
}

BOOL SetTaskMode(MSGPROC * proc, BYTE Flag)
{
	int i;
	struct _tag_tcb * tcb;

	for(i = 0; i < MAX_TCB; i++)
	{
		if(TCB[i].MsgProc == proc)
		{
			tcb = &TCB[i];
			if(tcb->Flag > MODE_TIMER)
			{
				switch(Flag){
				case MODE_NORMAL:
				case MODE_SEQ:
					tcb->Flag = Flag;
					return TRUE;
				}
			}
		}

	}
	return FALSE;
}

BYTE GetTaskMode(MSGPROC * proc)
{
	int i;

	for(i = 0; i < MAX_TCB; i++)
	{
		if(TCB[i].MsgProc == proc)
		{
			return TCB[i].Flag;
		}
	}
	return MODE_B_FULL;
}

static void ClearTask(TCBD * tcb)
{
	tcb->Flag     = MODE_NONE;
	tcb->MsgProc  = NULL;
	tcb->Next     = NULL;
	tcb->msg.top  = 0;
	tcb->msg.count= 0;
}

BOOL KillTask(MSG * msg)
{
	int i;
	TCBD * tcb;
	MSGDATA mdata;

	if(msg)
	{
		if(&root->msg == msg)
		{
			tcb = root;
			root  = root->Next;
			ClearTask(tcb);
		}
		else if(&troot->msg == msg)
		{
			tcb = root;
			root  = root->Next;
			ClearTask(tcb);
		}
		else for(i = 0; i < MAX_TCB; i++)
		{
			if(&TCB[i].Next->msg == msg)
			{
				tcb = TCB[i].Next;

				PostMessage(msg, MSG_DESTROY, 0);
				while(tcb->msg.count > 0)
				{
					if(GetMessage(msg, &mdata))
						SendMessage(msg, mdata.Message, mdata.Data);
				}
				TCB[i].Next = tcb->Next;
				ClearTask(tcb);
				return TRUE;
			}
		}
	}
	return FALSE;
}

MSGPROC * GetMsgFromProc(MSG * msg)
{
	int i;

	if(msg)
	for(i = 0; i < MAX_TCB; i++)
	{
		if(msg == &TCB[i].msg)
		{
			return TCB[i].MsgProc;
		}
	}
	return NULL;
}

MSG * GetProcFromMsg(MSGPROC * proc)
{
	int i;

	if(proc)
	for(i = 0; i < MAX_TCB; i++)
	{
		if(proc == TCB[i].MsgProc)
		{
			return &TCB[i].msg;
		}
	}
	return NULL;
}

MSG * GetDefTask(void)
{
	return DefCon;
}

void SetDefTask(MSG *msg)
{
	DefCon = msg;
}

/*******************************************************

	bZ[WAPI

 *******************************************************/

int SendMessage(MSG * msg, BYTE bMsg, long Data)
{
	return (GetMsgFromProc(msg))(msg, bMsg, Data);
}

BOOL PostMessage(MSG * msg, BYTE bMsg, long Data)
{
	int Index;
	BYTE BAK;

	BAK = get_ccr();
	DI;
	if(msg->count < MAX_MESSAGE)
	{
		Index = (msg->top + msg->count);
		if(Index >= MAX_MESSAGE) Index -= MAX_MESSAGE;

		msg->buf[Index].Message = bMsg;
		msg->buf[Index].Data    = Data;
		msg->count++;
		set_ccr(BAK);
		return TRUE;
	}
	set_ccr(BAK);
	return FALSE;
}

BOOL GetMessage(MSG * msg, MSGDATA * result)
{
	int Index;
	BYTE data;
	BYTE BAK;

	BAK = get_ccr();
	DI;
	if(msg->count > 0)
	{
		Index = msg->top;
		result->Message = msg->buf[Index].Message;
		result->Data    = msg->buf[Index].Data;
        Index++;
		if(Index >= MAX_MESSAGE) Index -= MAX_MESSAGE;
		msg->top = Index;
		msg->count--;
		set_ccr(BAK);
		return TRUE;
	}
	set_ccr(BAK);
	return FALSE;
}

extern struct tag_ADCData ADC_Data;

int DefaultProc(MSG * msg, BYTE bMsg, long Data)
{
	switch(bMsg){
	case MSG_THRDTERM:
		TermThread(Data);
		break;

	case MSG_AD1FINE:
		SetMoveAvg(0, Data);
		break;

	case MSG_AD2FINE:
		SetMoveAvg(1, Data);
		break;

	case MSG_AD3FINE:
		SetMoveAvg(2, Data);
		break;

	case MSG_AD4FINE:
		SetMoveAvg(3, Data);
		break;

	case MSG_AD5FINE:
		SetMoveAvg(4, Data);
		break;

	case MSG_AD6FINE:
		SetMoveAvg(5, Data);
		break;

	case MSG_AD7FINE:
		SetMoveAvg(6, Data);
		break;

	case MSG_AD8FINE:
		SetMoveAvg(7, Data);
		break;

	case MSG_QUIT:
        KillTask(msg);
		break;
	}
	return 0;
}

/*******************************************************

	XbhAPI

 *******************************************************/

#define THRD_NONE  0
#define THRD_START 1
#define THRD_SLEEP 2
#define THRD_RUN   3

THRDDATA THRD;

static long sp, spm;

static void init_thread(void)
{
	int i;

	THRD.Select = 0;
	for(i = 0; i < MAX_THRD; i++)
	{
		THRD.BLOCK[i].Flag  = THRD_NONE;
		THRD.BLOCK[i].Stack = NULL;
		THRD.BLOCK[i].proc  = NULL;
		THRD.BLOCK[i].IN    = DEV1;
		THRD.BLOCK[i].OUT   = DEV1;
	}
}

void ChangeThread(long *sp)
{
	int i;
	BLK *th;

	i = THRD.Select;

	th = &THRD.BLOCK[i];
	th->Stack = *(sp);
	th->IN  = GetSTDIN ();
	th->OUT = GetSTDOUT();

	if(THRD.BLOCK[i].Flag != THRD_START)
	{
		do {
			if(THRD.Count < 2)
			{
				i = 0;
			}
			else if(++i >= MAX_THRD)
			{
				if(root) i = 0;
				else     i = 1;
			}
		} while(THRD.BLOCK[i].Flag != THRD_RUN);

		if(THRD.Select != i)
		{
			th = &THRD.BLOCK[i];
			*(sp) = th->Stack;
			SetSTDIN (th->IN);
			SetSTDOUT(th->OUT);
			THRD.Select = i;
		}
	}
}

int MakeThread(THRDPROC * proc)
{
	int i, j;

	if(THRD.BLOCK[0].Flag == THRD_NONE)
	{
		THRD.BLOCK[0].Flag = THRD_RUN;

		THRD.Select = 0;
		THRD.Count++;
		THRD.BLOCK[0].IN  = GetSTDIN ();
		THRD.BLOCK[0].OUT = GetSTDOUT();
		CHG_THREAD;

		return 0;
	}
	else
	{
		for(i = 1; i < MAX_THRD; i++)
		{
			if(THRD.BLOCK[i].Flag == THRD_NONE)
			{
				sp = ThreadStckTop(i);
				DI;
				SET_SPONTER;
				THRD.BLOCK[i].Flag = THRD_START;
				THRD.BLOCK[i].proc = proc;
				j = THRD.Select;

				THRD.Select = i;
				THRD.Count++;

				THRD.BLOCK[i].IN  = GetSTDIN ();
				THRD.BLOCK[i].OUT = GetSTDOUT();

				CHG_THREAD;

				i = THRD.Select;

				if(THRD.BLOCK[i].Flag == THRD_RUN)
				{
					while(TRUE)
					{
						EI;
						if(THRD.BLOCK[i].Flag == THRD_RUN)
						{
							(THRD.BLOCK[i].proc)();
							SleepThread(i);
							PostMessage(GetDefTask(), MSG_THRDTERM, i);
						}
						CHG_THREAD;
					}
				}

				THRD.BLOCK[i].Flag = THRD_RUN;
				THRD.Select = j;

				RESET_SPONTER;
				EI;
				return i;
			}
		}
	}
	return -1;
}

void ResumeThread(int Select)
{
	if((Select >=0) && (Select < MAX_THRD))
	{
		if(THRD.BLOCK[Select].Flag == THRD_SLEEP)
			THRD.BLOCK[Select].Flag = THRD_RUN;
	}
}

void SleepThread(int Select)
{
	if((Select >=0) && (Select < MAX_THRD))
	{
		if(THRD.BLOCK[Select].Flag == THRD_RUN)
			THRD.BLOCK[Select].Flag = THRD_SLEEP;
		if(Select == THRD.Select) CHG_THREAD;
	}
}

void TermThread(int Select)
{
	BYTE BAK;

	if((Select >=0) && (Select < MAX_THRD))
	{
		BAK = get_ccr();
		DI;
		if(Select) switch(THRD.BLOCK[Select].Flag){
		case THRD_RUN:
		case THRD_SLEEP:
			THRD.Count--;
			THRD.BLOCK[Select].Flag = THRD_NONE;
			break;
		}
		set_ccr(BAK);
	}
}

int GetThreadMode(int Select)
{
	return THRD.BLOCK[Select].Flag;
}

/*******************************************************

	

 *******************************************************/

static DEVICE * dev[MAX_DEVICE];
STDIO STD;

static void init_dev(void)
{
	int i;

	STD.IN = STD_IN;
	STD.OUT= STD_OUT;

	for(i = 0; i < MAX_DEVICE; i++)
	{
		dev[i] = 0;
	}
}

BOOL SetSTDIN(int No)
{
	if(dev[No])
	{
		if(dev[No]->IRQ_ReadDevice)
		{
			STD.IN = No;
			return TRUE;
		}
	}
	return FALSE;
}

BOOL SetSTDOUT(int No)
{
	if(dev[No])
	{
		if(dev[No]->IRQ_WriteDevice)
		{
			STD.OUT= No;
			return TRUE;
		}
	}
	return FALSE;
}

int GetSTDIN(void)
{
	return STD.IN;
}

int GetSTDOUT(void)
{
	return STD.OUT;
}

void SetDevice(int No, DEVICE * Device)
{
	int i;

	if(Device)
	for(i = 0; i < 2; i++)
	{
		Device->Buffer[i].top   = 0;
		Device->Buffer[i].count = 0;
	}
	dev[No] = Device;
}

//---------------------------------------------------------------------------
//  [U[C^[tF[X
//---------------------------------------------------------------------------
char GetDEV(int No)
{
	BUF * Buf;
	char c;

	if(dev[No] != 0)
	{
		Buf = &dev[No]->Buffer[DEV_RDBUF];

		while(Buf->count == 0)
		{
			PolingDEV(No);
		}
		c = GetBuffer(Buf, MAX_BUFFER);
		return c;
	}
	else
		return 0;
}

void PutDEV(int No, char c)
{
	BUF * Buf;

	if(dev[No] != 0)
	{
		Buf = &dev[No]->Buffer[DEV_WRBUF];

		while(Buf->count >= MAX_BUFFER)
		{
			PolingDEV(No);
		}
		PutBuffer(Buf, c, MAX_BUFFER);
		PolingDEV(No);
	}
}

void ControlDEV(int No, char *data)
{
	if(dev[No] != 0)
	{
		dev[No]->Ctrl(data);
	}
}

char EmptyReadBuf(int No)
{
	if(dev[No] != 0)
	{
		if(dev[No]->Buffer[DEV_RDBUF].count >0)
			return FALSE;
	}
	return TRUE;
}

char EmptyWriteBuf(int No)
{
	if(dev[No] != 0)
	{
		if(dev[No]->Buffer[DEV_WRBUF].count > 0)
			return FALSE;
	}
	return TRUE;
}

void PrintDEV(int No, char * msg)
{
	int i;
	char c;

	if(dev[No] != 0)
	{
		i = 0;
		c = msg[i];
		while(c != 0)
		{
			while(dev[No]->Buffer[DEV_WRBUF].count == MAX_BUFFER)
				PolingDEV(No);
			PutBuffer(&dev[No]->Buffer[DEV_WRBUF], c, MAX_BUFFER);
			i++;
			c = msg[i];
		}
		PolingDEV(No);
	}
}

void PolingDEV(int No)
{
	BYTE BAK;

	if(dev[No] != 0)
	{
		BAK = get_ccr();
		DI;
		if(dev[No]->IRQ_ReadDevice) IRQ_ReadDEV(No);
		if(!EmptyWriteBuf(No))
		{
			if(dev[No]->IRQ_WriteDevice) IRQ_WriteDEV(No);
		}
		set_ccr(BAK);
	}
}
//---------------------------------------------------------------------------
//  hCoC^[tF[X
//---------------------------------------------------------------------------

void IRQ_ReadDEV(int No)
{
	char c;
	DEVICE * D;

	D = dev[No];
	if(D->IRQ_ReadDevice)
	{
		if((D->RD_Ready == 0) ||
		  ((D->RD_Ready != 0) && (D->RD_Ready)()))
		{
			c = (D->IRQ_ReadDevice)();
			PutBuffer(&D->Buffer[DEV_RDBUF], c, MAX_BUFFER);
			if(STD.IN == No)
				PostMessage(GetDefTask(), MSG_ONSTDIN, c);
		}
	}
	else
		PutBuffer(&D->Buffer[DEV_RDBUF], 0, MAX_BUFFER);

}

void IRQ_WriteDEV(int No)
{
	DEVICE * D;
	char c;

	D = dev[No];
	if(D->IRQ_WriteDevice)
	{
		if((D->WR_Ready == 0) ||
		  ((D->WR_Ready != 0) && (D->WR_Ready)()))
		{
			c = GetBuffer(&D->Buffer[DEV_WRBUF], MAX_BUFFER);
			(D->IRQ_WriteDevice)(c);
		}
	}
	else
	{
		GetBuffer(&D->Buffer[DEV_WRBUF], MAX_BUFFER);
	}
}

char GetCh(void)
{
	return GetDEV(STD.IN);
}

void PutCh(char c)
{
	PutDEV(STD.OUT, c);
}

void print(char * data)
{
	PrintDEV(STD.OUT, data);
}

void println(char * data)
{
	 PrintDEV(STD.OUT, data);
	 PrintDEV(STD.OUT, "\r\n");
}

/***************************************************

	rv͊֐

 ***************************************************/
static struct {
	unsigned char data;
	unsigned char old;
	unsigned char flag;
	union{
		struct{
			unsigned clk1 : 2;
			unsigned clk2 : 2;
			unsigned clk3 : 2;
			unsigned clk4 : 2;
		}SEP;
		BYTE ALL;
	}CLK;
	unsigned char wait;
} SW;

void SW_Ancer(BYTE NO, BYTE mask, BYTE data, BYTE Msg_ON, BYTE Msg_OFF)
{
	if(SW.flag & mask)
	{
		if(data & mask)
		{
			if(SW.wait == 0)
			{
				SW.wait = LONG_PUSH;
				SW.CLK.ALL = 0;
			}
			PostMessage(GetDefTask(), Msg_OFF, 0);
		}
		else
		{
			switch(NO){
			case 0: SW.CLK.SEP.clk1++; break;
			case 1: SW.CLK.SEP.clk2++; break;
			case 2: SW.CLK.SEP.clk3++; break;
			case 3: SW.CLK.SEP.clk4++; break;
//			case 4: SW.CLK.SEP.clk5++; break;
//			case 5: SW.CLK.SEP.clk6++; break;
//			case 6: SW.CLK.SEP.clk7++; break;
//			case 7: SW.CLK.SEP.clk8++; break;
			}
			PostMessage(GetDefTask(), Msg_ON, 0);
		}
	}
}

static void sw_proc(void)
{
	BYTE data;

	data = GetSW();
	if(SW.old == data)
	{
		if(SW.data != SW.old)
		{
			SW.flag = SW.data ^ SW.old;
			SWEvent(data);
			SW.data = data;
		}
	}
	else
		SW.old = data;
}

/***************************************************

	PWM LEDhCu`oh

 ***************************************************/
static struct {
	char attr;
	unsigned char count;
	unsigned char data;
} LED;

static void led_proc(void)
{
	if(	LED.attr < 9)
	{
		LED.count = (LED.count + 1) & 0x07;
		if(LED.count >= LED.attr)
			WriteLEDPort(0x00);
		else
			WriteLEDPort(LED.data);
	}
}

void SetLED(char atr, char data)
{
	LED.attr = atr;
	LED.data = data;
}

/***************************************************

	^C}[ݒ^XN̎s

 ***************************************************/
#define LOOP_CLEAR 0
#define LOOP_SET   1
#define LOOP_QUIT  2

static void ExecuteProc(void)
{
	TCBD *tcb;
	BYTE FLAG;
	MSGDATA	data;

	tcb = troot;
	FLAG = LOOP_CLEAR;

	if(tcb) do
	{
		if(tcb->msg.count)
		{
			if(GetMessage(&tcb->msg, &data))
			{
				(tcb->MsgProc)(&tcb->msg, data.Message, data.Data);
				FLAG = LOOP_SET;
			}
		}

		if(tcb->Next)
			tcb = tcb->Next;
		else
		{
			if(FLAG == LOOP_CLEAR)
				FLAG = LOOP_QUIT;
			else
			{
				tcb = troot;
				FLAG = LOOP_CLEAR;
			}
		}

	}
	while((FLAG != LOOP_QUIT) || tcb->Next);
}

/***************************************************

	^C}[֐

 ***************************************************/

void SystemTimer(void)
{
	led_proc();
}

void Timer100mSec(void)
{
	BYTE FLAG;
	MSG *DefCon;

	if(SW.wait > 0)
	{
		DefCon = GetDefTask();
		
		if(SW.wait == 1)
		{
			if((SW.data & SW_MASK) == SW_MASK)
				PostMessage(DefCon, MSG_SWLONG, 0);
			else
			{
				if(SW.data & SW1_MASK)
					PostMessage(DefCon, MSG_SW1LONG, 0);
				if(SW.data & SW2_MASK)
					PostMessage(DefCon, MSG_SW2LONG, 0);
				if(SW.data & SW3_MASK)
					PostMessage(DefCon, MSG_SW3LONG, 0);
				if(SW.data & SW4_MASK)
					PostMessage(DefCon, MSG_SW4LONG, 0);
			}
			SW.wait = 0;
			SW.CLK.ALL = 0;
		}
		else
		{
			SW.wait--;
			FLAG = 0;
			if(SW.CLK.SEP.clk1 > 1)
			{
				PostMessage(DefCon, MSG_SW1DCLK, 0);
				FLAG = 1;
			}
			if(SW.CLK.SEP.clk2 > 1)
			{
				PostMessage(DefCon, MSG_SW2DCLK, 0);
				FLAG = 1;
			}
			if(SW.CLK.SEP.clk3 > 1)
			{
				PostMessage(DefCon, MSG_SW3DCLK, 0);
				FLAG = 1;
			}
			if(SW.CLK.SEP.clk4 > 1)
			{
				PostMessage(DefCon, MSG_SW4DCLK, 0);
				FLAG = 1;
			}

			if(SW.wait < (LONG_PUSH - 8))
			{
				if(SW.CLK.SEP.clk1 == 1)
				{
					PostMessage(DefCon, MSG_SW1CLCK, 0);
					FLAG = 1;
				}
				if(SW.CLK.SEP.clk2 == 1)
				{
					PostMessage(DefCon, MSG_SW2CLCK, 0);
					FLAG = 1;
				}
				if(SW.CLK.SEP.clk3 == 1)
				{
					PostMessage(DefCon, MSG_SW3CLCK, 0);
					FLAG = 1;
				}
				if(SW.CLK.SEP.clk4 == 1)
				{
					PostMessage(DefCon, MSG_SW4CLCK, 0);
					FLAG = 1;
				}
			}
			if(FLAG)
			{
				SW.wait = 0;
				SW.CLK.ALL = 0;
			}
		}
	}
}

/***************************************************

	^C}[荞API

 ***************************************************/
struct {
	MSG * msg;
	long data;
	int count;
	int interval;
} TIMER[MAX_TIMER];

static void init_ptimer(void)
{
	int i;

	for(i = 0; i < MAX_TIMER; i++)
	{
		TIMER[i].msg   = 0;
		TIMER[i].count = 0;
		TIMER[i].interval = 0;
		TIMER[i].data  = 0;
	}
}

int SetTimer(MSG * msg, int WaitTime, long data)
{
	int i;
	BYTE BAK;

	BAK = get_ccr();
	DI;
	for(i = 0; i < MAX_TIMER; i++)
	{
		if(TIMER[i].interval==0)
		{
			TIMER[i].msg   = msg;
			TIMER[i].count = 0;
			TIMER[i].interval = WaitTime;
			TIMER[i].data  = data;
			EI;
			return i;
		}
	}
	set_ccr(BAK);
	return -1;
}

void KillTimer(int ID)
{
	BYTE BAK;

	BAK = get_ccr();
	DI;
	TIMER[ID].interval = 0;
	TIMER[ID].msg   = 0;
	TIMER[ID].data  = 0;
	TIMER[ID].count = 0;
	set_ccr(BAK);
}

void Timer10mSec(void)
{
	int i;

	for(i = 0; i < MAX_TIMER; i++)
	{
		if(TIMER[i].interval)
		{
			TIMER[i].count++;
			if(TIMER[i].count >= TIMER[i].interval)
			{
				TIMER[i].count -= TIMER[i].interval;
				PostMessage(TIMER[i].msg, MSG_TIMEUP, TIMER[i].data);
			}
		}
	}
	sw_proc();
	ExecuteProc();
}

/***************************************************

	Z[t[h֐

 ***************************************************/
 static int SafeMode(MSG * msg, BYTE Msg, long Data)
{
	static int ID[3], mode;
	int t;

	switch(Msg){
	case MSG_INIT:
		WriteLEDPort(12);
		SetLED(1,0x0F);
		mode = 0;
		break;

	case MSG_SW1CLCK:
		mode = (mode + 1) % (MAX_DEVICE+2);
		SetLED(1, mode-1);
		break;

	case MSG_SW2CLCK:
		switch(mode){
		case 0:
        	root = 0;
			KillTask(msg);
			break;

		case 1:
			SetLED(8, mode-1);
			WriteLEDPort(mode-1);
			init_bios();
			break;

		default:
			t = mode-2;
			if(dev[t])
			{
				SetLED(8, mode-1);
				WriteLEDPort(mode-1);
				(dev[t]->Init)();
				SetSTDIN(t);
				SetSTDOUT(t);
			}
		}
		break;

	case MSG_SW2DCLK:
		t = GetSTDOUT()+1;
		SetLED(4, t);
		WriteLEDPort(~t);

		if((mode > 1) && (mode < MAX_DEVICE+2))
		{
			if(dev[mode-2])
			{
				println("Device OK");
			}
		}
		break;

	case MSG_SW2LONG:
		t = GetSTDOUT()+1;
		SetLED(4, t);
		WriteLEDPort(~t);

		if((mode > 1) && (mode < MAX_DEVICE+2))
		{
			if(dev[GetSTDIN()]->IRQ_ReadDevice)
			{
				do{
					t = (int)GetCh();
					PutCh((char)t);
				} while(t != '\n');
			}
		}
		break;
	}
}

/***************************************************

	֐

 ***************************************************/
static void init(void)
{
	int i;
	MSGDATA data;

	init_task();
	init_thread();
	init_ptimer();
	RegistDriver();

	init_io();

//	`FbNpR[h
	while(GetDIPSW() & SW1_MASK)
	{
		WriteLEDPort(GetSW());
	}
	WriteLEDPort(15);

//  Xg[foCXhCȏgݍ
	if(GetDIPSW() & SW2_MASK)
	{
		SetDefTask(MakeTask(SafeMode, MODE_NORMAL));
		WriteLEDPort(14);
		init_systimer();
		WriteLEDPort(13);
		SetLED(8,13);
		EI;
		while(root)
		{
			if(GetMessage(GetDefTask(), &data))
				(root->MsgProc)(GetDefTask(), data.Message, data.Data);
			PoleIO();
		}
		DI;
	}
	else 
	{
		init_bios();
		for(i = MAX_DEVICE-1; i >= 0; i--)
		{
			if(dev[i] != 0)
			{
				if(dev[i]->Init != 0)
				{
					WriteLEDPort(i+1);
					(dev[i]->Init)();
					SetSTDIN(i);
					SetSTDOUT(i);
				}
			}
		}
		init_systimer();
	}
	SetLED(8,0);
	PutCh('\f');
	println("SROS Ver0.2");
}

int main(void)
{
	char Flag;
	MSGDATA data;
	TCBD * task = NULL;

	DI;
	init();

	MakeThread(NULL);
	SetDefTask(MakeTask(ShellMain, MODE_NORMAL));

	EI;

	while(TRUE)
	{
		while(task)
		{
			if((task->Flag == MODE_SEQ) ||
			   (task->msg.count > 0))
			{
				Flag = 1;
				if(GetMessage(&task->msg, &data))
				{
					SetSTDIN (task->IN);
					SetSTDOUT(task->OUT);
					(task->MsgProc)
						(&task->msg, data.Message, data.Data);
					task->IN  = GetSTDIN();
					task->OUT = GetSTDOUT();
				}
			}
			task = task->Next;
		}

		task = root;
		PoleIO();

		if(THRD.Count > 1)
		{
			if(Flag == 0) CHG_THREAD;
		}
		Flag = 0;
	}
}
