//---------------------------------------------------------------------------

#include "SROS.h"
#include "OSCore.h"
#include "BIOS.h"
//---------------------------------------------------------------------------

/*******************************************************

	Oobt@[API

 *******************************************************/

BOOL PutBuffer(BUF *buf, BYTE data, int size)
{
	int Index;
	BYTE BAK;

	BAK = get_ccr();
	DI;

	if(buf->count < MAX_BUFFER)
	{
		Index = buf->top + buf->count;
		if(Index >= size) Index -= size;
		buf->buf[Index] = data;
		buf->count++;
		set_ccr(BAK);
		return TRUE;
	}
	set_ccr(BAK);
	return FALSE;
}

BYTE GetBuffer(BUF *buf, int size)
{
	BYTE data;
	BYTE BAK;

	BAK = get_ccr();
	DI;

	if(buf->count > 0)
	{
		data = buf->buf[buf->top];
		buf->top = buf->top + 1;
		if(buf->top >= size) buf->top -= size;
		buf->count--;
		set_ccr(BAK);
		return data;
	}
	set_ccr(BAK);
	return 0;
}

int StrPos(char *data, char *subdata)
{
	BYTE i, j, k;

	for(k = 0; k < 255; k++)
	{
		if(subdata[k] == 0) break;
	}
	
	for(i = 0; data[i]; i++)
	{
		if(data[i] == subdata[0])
		{
			for(j = 0; j < k; j++)
			{
				if(data[i+j] != subdata[j])
				{
					break;
				}
			}
			if(j == k)
			{
				return i;
			}
		}
	}
	return -1;
}

int MakeList(char *data, char **list, char sep)
{
	int i, j;

	j = 1;
	list[0] = data;
	for(i = 0; data[i] ; i++)
	{
		if(data[i] == sep)
		{
			data[i] = 0;
			list[j] = &data[i+1];
			j++;
		}
	}
}

static void PutLeftCode(void)
{
	PutCh(0x1B); PutCh(0x5B); PutCh(0x44);
}

static void PutRightCode(void)
{
	PutCh(0x1B); PutCh(0x5B); PutCh(0x43);
}

void MakeCmdText(CMD_BUF * COMMAND)
{
	char ch;
	int state = 0;

	ch = GetCh();
	switch(state){
	case 0:
		switch(ch){
		case 0x08:
			if(COMMAND->count > 0)
			{
				PutLeftCode();
				PutCh(' ');
				PutLeftCode();
				COMMAND->text[--COMMAND->count] = 0x00;
			}
			break;

		case 0x0A: case 0x0D:
			PutCh(ch);
			println("");
			if(COMMAND->count > 0)
			{
				PostMessage(GetDefTask(), MSG_COMMAND, COMMAND->count);
				COMMAND->count = 0;
			}
			break;

		case 0x1B:
			PutCh(ch);
			state = 1;
			break;

		default:
			PutCh(ch);
			COMMAND->text[COMMAND->count] = ch;

			COMMAND->text[++COMMAND->count] = 0x00;
		}
		break;

	case 1:
		switch(ch){
		case 0x5B:
			PutCh(ch);
			state = 2;
			break;
		default:
			state = 0;
		}
		break;

	case 2:
		switch(ch){
		case 0x44:
			PutCh(ch);
			if(COMMAND->count > 0)
			{
				PutLeftCode();
				COMMAND->count--;
			}
			break;

		case 0x43:
			PutCh(ch);
			if(COMMAND->count < (MAX_CMDCHAR-1))
			{
				if(COMMAND->text[COMMAND->count]==0x00)
				{
					COMMAND->text[COMMAND->count] = ' ';
					COMMAND->text[COMMAND->count+1]   = 0x00;
				}
				PutCh(COMMAND->text[COMMAND->count++]);
			}
			break;
		}
		break;
	}
}

void MakeField(char *data, char **list, int *count)
{
	int i, j, k;

	i = 1;
	j = 0;
	k = 0;

	list[0] = data;

	while(data[i])
	{
		if(j == count[k])
		{
			j = 0;
			k++;
		}
		list[k] = &data[i];
		data[i] = 0;
		i++;
		j++;
	}
}

BOOL IsHex(char * data)
{
	int i;

	i = 0;
	while((data[i] != 0x00) && (data[i] != 0x0A) && (data[i] != 0x0D)  && (data[i] != ' '))
	{
		if(i > 9)						 return FALSE;
		if(data[i] < '0')				 return FALSE;
		if(data[i] <= '9')				 i++;
		else if(data[i] < 'A')			 return FALSE;
		else if(((data[i] & 0xDF) > '@')
			 && ((data[i] & 0xDF) < 'G'))i++;
		else							 return FALSE;
	}
	if(i > 0) return TRUE;
	else	  return FALSE;
}

static BYTE HexToDec(char data)
{
	if     (data <  '0') return 0;
	else if(data >  'F') return 0;
	else if(data <= '9')
	{
		return data - 0x30;
	}
	else if(data >  '@')
	{
		return data -0x37;
	}
	else
		return 0;
}

BYTE HexToByte(char * hexdata)
{
	int i;

	i = strlen(hexdata);
	if(i > 1)
		return (HexToDec(hexdata[0])<<4)+ HexToDec(hexdata[1]);
	else 
		return HexToDec(hexdata[0]);
}

int HexToInt(char * hexdata)
{
	int data, pos;
	char buf[10];

	strcpy(buf, hexdata);
	data = 0;
	while(buf[0] != 0)
	{
		if(strlen(buf) > 1)
			data = (data << 8) + HexToByte(buf);
		else
			data = (data << 4) + HexToByte(buf);
		pos = 0;
		if(strlen(buf) < 2) buf[pos] = 0;
		else while(buf[pos] != 0)
		{
			buf[pos] = buf[pos+2];
			pos++;
		}
	}
	return data;
}

void PutHex(BYTE value)
{
	char tmp[3];

	tmp[0] = ((value >> 4) & 0x0F);
	if(tmp[0] > 9) tmp[0] = tmp[0]+'7';
	else		   tmp[0] = tmp[0]+'0';

	tmp[1] = ((value     ) & 0x0F);
	if(tmp[1] > 9) tmp[1] = tmp[1]+'7';
	else		   tmp[1] = tmp[1]+'0';

	tmp[2] = 0;
	print(tmp);
}


