//---------------------------------------------------------------------------
#include "SROS.h"
#include "OSCore.h"
#include "BIOS.h"
#include "Device1.h"
#include "Device2.h"
#include "Device3.h"
#include "Device4.h"

//---------------------------------------------------------------------------

#define SCI_DEV DEV1
#define LCD_DEV DEV2

#define LED_PORT PB.DR.BYTE
#define KSW_PORT PA.DR.BYTE
#define DSW_PORT PA.DR.BYTE

#define SYS_TIMER ITU2
#define THRD_TIMER ITU2
#define SYS_TIMER_STR ITU. TSTR.BIT.STR2

void RegistDriver(void)
{
	AddSCI(SCI_DEV);
	AddLCD(LCD_DEV);
//    AddPaipe(DEV3);
//	AddDEV4(DEV4);
}

void init_io(void)
{
	BSC.ABWCR.BYTE	= 0x06;		/* 8bit BUS MODE */

	P1.DDR 			= 0xff;		/* all OUT 			*/
	P2.DDR 			= 0xff;		/* all OUT			*/
	P2.PCR.BYTE 	= 0x00;		/* Pull up off		*/
	P5.DDR 			= 0xff;		/* all OUT			*/
	P5.PCR.BYTE 	= 0x00;		/* Pull up off		*/
	P6.DDR 			= 0xff;		/* all OUT			*/
	P9.DDR 			= 0xff;		/* all OUT			*/
	P8.DDR 			= 0xff;		/* all OUT			*/
	PA.DDR 			= 0xf0;		/* bit7..4 out , bit3..0 in	*/
	PB.DDR 			= 0xff;		/* bit7..0 out  */
	ITU.TSTR.BYTE	= 0x00;
}

void init_systimer(void)
{
	SYS_TIMER_STR			= 0;
	SYS_TIMER.GRA			= 5000;
	SYS_TIMER.GRB			= 2500;
	SYS_TIMER.TCR. BYTE		= 0x23;
	SYS_TIMER.TSR. BYTE		= 0x00;
	SYS_TIMER.TIER.BYTE		= 0x03;
	SYS_TIMER_STR			= 1;
}

void init_adc(void);

void init_bios(void)
{
	init_adc();
}

/*****************************************************

	VXehn|[g̃C^[tF[X

 *****************************************************/
void PoleIO(void)
{
	PolingDEV(LCD_DEV);
}

/*==========================================================================
							ADC hCo
==========================================================================*/
struct tag_ADCData ADC_Data;

#define ADCSR AD.ADCSR

static void InitADData(int index, WORD data)
{
	int i;

	ADC_Data.data[index] = data;
	ADC_Data.SUM[index] = 0;
	for (i = 0; i < 8; i++)
	{
		ADC_Data.AVGDATA[index][i]= data;
		ADC_Data.SUM[index] = ADC_Data.SUM[index] + data;
	}
}

static void StartADC(int Select)
{
	int i;

	if( Select < 4) ADCSR.BIT.SCAN = 0;
	else            ADCSR.BIT.SCAN = 1;
	ADCSR.BIT.CH = Select;
	ADCSR.BYTE = ADCSR.BYTE | 0x20;
	ADC_Data.Select = Select;
}

void SetMoveAvg(int Select, WORD data)
{
	int temp;

	ADC_Data.SUM[Select] = ADC_Data.SUM[Select]
						 - ADC_Data.AVGDATA[Select][ADC_Data.Count]
						 + data;
	ADC_Data.AVGDATA[Select][ADC_Data.Count] = data;
	temp = ADC_Data.SUM[Select];
	temp = temp >> 3;
	ADC_Data.data[Select] = temp;
}

void init_adc(void)
{
	int i;

	ADCSR.BIT.ADST = 0;
	ADCSR.BYTE    = ADCSR.BYTE | 0x58;
	for(i = 0; i < 8; i++)
	{
		InitADData(i, 0);
	}
}

void int_adi()  __attribute__ ((interrupt_handler));
void int_adi(void)
{
	ADCSR.BYTE    = (ADCSR.BYTE & 0x58) | 0x47;
	switch(ADC_Data.Select){
	case 0:
		PostMessage(GetDefTask(), MSG_AD1FINE, AD.ADDRA);
		break;

	case 1:
		PostMessage(GetDefTask(), MSG_AD2FINE, AD.ADDRB);
		break;

	case 2:
		PostMessage(GetDefTask(), MSG_AD3FINE, AD.ADDRC);
		break;

	case 3:
		PostMessage(GetDefTask(), MSG_AD4FINE, AD.ADDRD);
		break;

	case 4:
		PostMessage(GetDefTask(), MSG_AD5FINE, AD.ADDRA);
		break;

	case 5:
		PostMessage(GetDefTask(), MSG_AD6FINE, AD.ADDRB);
		break;

	case 6:
		PostMessage(GetDefTask(), MSG_AD7FINE, AD.ADDRC);
		break;

	case 7:
		PostMessage(GetDefTask(), MSG_AD8FINE, AD.ADDRD);
		break;
	}
}

/*==========================================================================
							LED hCo
==========================================================================*/
void WriteLEDPort(BYTE value)
{
	BYTE temp;

	temp = LED_PORT & 0xF0;
	LED_PORT = temp | (~value & 0x0F);
}

/*==========================================================================
							rv hCo
==========================================================================*/

BYTE GetSW(void)
{
	return ~KSW_PORT & SW_MASK;
}

BYTE GetDIPSW(void)
{
	return ~DSW_PORT & SW_MASK;
}

//	XCb`Cxg̊g

void SWEvent(BYTE data)
{
	SW_Ancer( 0, SW1_MASK, data, MSG_SW1ON, MSG_SW1OFF);
	SW_Ancer( 1, SW2_MASK, data, MSG_SW2ON, MSG_SW2OFF);
	SW_Ancer( 2, SW3_MASK, data, MSG_SW3ON, MSG_SW3OFF);
	SW_Ancer( 3, SW4_MASK, data, MSG_SW4ON, MSG_SW4OFF);
//	SW_Ancer( 4, SW5_MASK, data, MSG_SW5ON, MSG_SW5OFF);
//	SW_Ancer( 5, SW6_MASK, data, MSG_SW6ON, MSG_SW6OFF);
//	SW_Ancer( 6, SW7_MASK, data, MSG_SW7ON, MSG_SW7OFF);
//	SW_Ancer( 7, SW8_MASK, data, MSG_SW8ON, MSG_SW8OFF);
}

/*****************************************************

	荞ݏ

 *****************************************************/
/*==========================================================================
							^C}[荞
==========================================================================*/
void int_imia2()  __attribute__ ((interrupt_handler));
void int_imib2()  __attribute__ ((interrupt_handler));

static long sp;
extern THRDDATA THRD;

#define STACK_SIZE 394
static struct {
	int mem[STACK_SIZE];
} stackbuf;

static void ClearStack(void)
{
	int i;

	for(i = 0; i < STACK_SIZE; i++) stackbuf.mem[i] = 0;
}

void DispStackMem(int index)
{
	int i, j;

	for(i = 0; i < STACK_SIZE; i += 16)
	{
		for(j = 0; j < 16; j++)
		{
			PutHex(stackbuf.mem[i+j] >> 8);
			PutHex(stackbuf.mem[i+j] & 0x0FF);
			PutCh(' ');
		}
		println("");
	}
}

long ThreadStckTop(int Select)
{
	switch(Select){
	case 1:
		return (long) &stackbuf.mem[STACK_SIZE];
	case 2:
		return (long) &stackbuf.mem[STACK_SIZE-128];
	case 3:
		return (long) &stackbuf.mem[STACK_SIZE-256];
	}
}

void reset_TTimer(void)
{
	THRD_TIMER.GRB = THRD_TIMER.TCNT-1;
	if(THRD_TIMER.GRB > SYS_TIMER.GRA)
	{
		THRD_TIMER.GRB = THRD_TIMER.GRB % SYS_TIMER.GRA;
	}
}

void int_imia2 (void)
{
	static int count=0;

	SYS_TIMER.TSR.BIT.IMFA = 0;

	switch(++count){
	case  20:
		Timer100mSec();
		count = 0;
	}
	if(count & 1)
		Timer10mSec();
	SystemTimer();
}

void int_imib2(void)
{
__asm__(
"	push.l	er1\n"
"	push.l	er2\n"
);
	THRD_TIMER.TSR.BIT.IMFB = 0;

	if(THRD.Count > 1)
	{
__asm__(
"	push.l	er3\n"
"	push.l	er4\n"
"	push.l	er5\n"
"	push.l	er6\n"
"	mov.l	er7,@_sp\n"
);
		reset_TTimer();
		ChangeThread(&sp);
__asm__(
"	mov.l	@_sp,er7\n"
"	pop.l	er6\n"
"	pop.l	er5\n"
"	pop.l	er4\n"
"	pop.l	er3\n"
);
	}
__asm__(
"	pop.l	er2\n"
"	pop.l	er1\n"
);
}
/*==========================================================================
							gbv
==========================================================================*/
void int_trap0()  __attribute__ ((interrupt_handler));
void int_trap0(void)
{
__asm__(
"	push.l	er1\n"
"	push.l	er2\n"
);
	if(THRD.Count > 1)
	{
__asm__(
"	push.l	er3\n"
"	push.l	er4\n"
"	push.l	er5\n"
"	push.l	er6\n"
"	mov.l	er7,@_sp\n"
);
		reset_TTimer();
		ChangeThread(&sp);
__asm__(
"	mov.l	@_sp,er7\n"
"	pop.l	er6\n"
"	pop.l	er5\n"
"	pop.l	er4\n"
"	pop.l	er3\n"
);
	}
__asm__(
"	pop.l	er2\n"
"	pop.l	er1\n"
);
}

