/*======================================================================
							N9604 Address								
======================================================================*/
#define		USB9602R			(*(volatile unsigned char *)0x400003)
#define		USB9602D			(*(volatile unsigned char *)0x400001)

/*======================================================================
							N9604 Define								
======================================================================*/
#define		USB_CLKDIV			0x04				/* CLKOUT = 48MHz/4 = 12MHz */


/* USB1.0NGXg */
#define		USB_GET_STATUS			0
#define		USB_CLEAR_FEATURE		1
#define		USB_SET_FEATURE			3
#define		USB_SET_ADDRESS			5
#define		USB_GET_DESCRIPTOR		6
#define		USB_SET_DESCRIPTOR		7
#define		USB_GET_CONFIGURATION	8
#define		USB_SET_CONFIGURATION	9
#define		USB_GET_INTERFACE		10
#define		USB_SET_INTERFACE		11
#define		USB_SYNCH_FRAME			12

/* fBXNv^ */
#define		USB_DEVICE				1
#define		USB_CONFIGURATION		2
#define		USB_XSTRING				3
#define		USB_INTERFACE			4
#define		USB_ENDPOINT			5
#define		USB_HID					0x21
#define		USB_HIDREPORT			0x22
#define		USB_HIDPHYSICAL			0x23

/* HIDNGXg */
#define		USB_GET_REPORT			0x01
#define		USB_GET_IDLE			0x02
#define		USB_GET_PROTOCOL		0x03
#define		USB_SET_REPORT			0x09
#define		USB_SET_IDLE			0x0A
#define		USB_SET_PROTOCOL		0x0B


/*======================================================================
							N9604 Register								
======================================================================*/
#define		USB_MCNTRL			0x00	/*Main control register   */
#define		USB_CCONF			0x01	/*Clk. config. register   */
#define		USB_TCR				0x02	/*Xcvr config. register   */
#define		USB_RID				0x03	/*Rev. ID      register   */
#define		USB_FAR				0x04	/*Func address register   */
#define		USB_NFSR			0x05	/*Node func st register   */
#define		USB_MAEV			0x06	/*Main event   register   */
#define		USB_MAMSK			0x07	/*Main mask    register   */
#define		USB_ALTEV			0x08	/*Alt. event   register   */
#define		USB_ALTMSK			0x09	/*ALT  mask    register   */
#define		USB_TXEV			0x0A	/*TX   event   register   */
#define		USB_TXMSK			0x0B	/*TX   mask    register   */
#define		USB_RXEV			0x0C	/*RX   event   register   */
#define		USB_RXMSK			0x0D	/*RX   mask    register   */
#define		USB_NAKEV			0x0E	/*NAK  event   register   */
#define		USB_NAKMSK			0x0F	/*NAK  mask    register   */
#define		USB_FWEV			0x10	/*FIFO warning register   */
#define		USB_FWMSK			0x11	/*FIFO warning mask       */
#define		USB_FNH				0x12	/*Frame nbr hi register   */
#define		USB_FNL				0x13	/*Frame nbr lo register   */
#define		USB_DMACNTRL		0x14	/*DMA  control register   */

#define		USB_EPC0			0x20	/*Endpoint0    register   */
#define		USB_TXD0			0x21	/*TX   data    register 0 */
#define		USB_TXS0			0x22	/*TX   status  register 0 */
#define		USB_TXC0			0x23	/*TX   command register 0 */

#define		USB_RXD0			0x25	/*RX   data    register 0 */
#define		USB_RXS0 			0x26	/*RX   status  register 0 */
#define		USB_RXC0			0x27	/*RX   command register 0 */

#define		USB_EPC1			0x28	/*Endpoint1    register   */
#define		USB_TXD1			0x29	/*TX   data    register 1 */
#define		USB_TXS1			0x2A	/*TX   status  register 1 */
#define		USB_TXC1			0x2B	/*TX   command register 1 */

#define		USB_EPC2			0x2C	/*Endpoint2    register   */
#define		USB_RXD1			0x2D	/*RX   data    register 1 */
#define		USB_RXS1			0x2E 	/*RX   status  register 1 */
#define		USB_RXC1			0x2F	/*RX   command register 1 */

#define		USB_EPC3			0x30	/*Endpoint3    register   */
#define		USB_TXD2			0x31 	/*TX   data    register 2 */
#define		USB_TXS2			0x32	/*TX   status  register 2 */
#define		USB_TXC2			0x33 	/*TX   command register 2 */

#define		USB_EPC4			0x34	/*Endpoint4    register   */
#define		USB_RXD2			0x35	/*RX   data    register 2 */
#define		USB_RXS2			0x36	/*RX   status  register 2 */
#define		USB_RXC2			0x37	/*RX   command register 2 */

#define		USB_EPC5			0x38	/*Endpoint5    register   */
#define		USB_TXD3			0x39	/*TX   data    register 3 */
#define		USB_TXS3			0x3A  	/*TX   status  register 3 */
#define		USB_TXC3			0x3B	/*TX   command register 3 */
                                        
#define		USB_EPC6			0x3C	/*Endpoint6    register   */
#define		USB_RXD3			0x3D	/*RX   data    register 3 */
#define		USB_RXS3			0x3E	/*RX   status  register 3 */
#define		USB_RXC3			0x3F	/*RX   command register 3 */

/*---------- MCNTRL bits -------------*/
#define		USB_SRST			0x01	/*software reset          */
#define		USB_DBG				0x02    /*debug mode              */
#define		USB_VGE				0x04 	/*voltage regulator enable*/
#define		USB_NAT				0x08	/*node attached           */
#define		USB_INT_DIS			0x00    /*interrupts disabled     */
#define		USB_INT_L_O			0x40	/*act lo ints, open drain */
#define		USB_INT_H_P			0x80	/*act hi ints, push pull  */
#define		USB_INT_L_P			0xC0	/*act lo ints, push pull  */

/*------------ FAR bits --------------*/
#define		USB_AD_EN			0x80	/*address enable          */

/*------------ NFSR bits -------------*/
#define		USB_RST_ST			0x00	/*reset       state       */
#define		USB_RSM_ST			0x01	/*resume      state       */
#define		USB_OPR_ST			0x02	/*operational state       */
#define		USB_SUS_ST			0x03	/*suspend     state       */

/*-------- MAEV, MAMSK bits ---------*/
#define		USB_WARN			0x01	/*warning bit has been set*/
#define		USB_ALT				0x02	/*alternate event         */
#define		USB_TX_EV			0x04	/*transmit event          */
#define		USB_FRAME			0x08	/*SOF packet received     */
#define		USB_NAK				0x10	/*NAK event               */
#define		USB_ULD				0x20	/*unlock locked detected  */
#define		USB_RX_EV			0x40	/*receive event           */
#define		USB_INTR_E			0x80	/*master interrupt enable */

/*-------- ALTEV, ALTMSK bits --------*/
#define		USB_EOP				0x08	/*end of packet           */
#define		USB_SD3				0x10	/*3 ms suspend            */
#define		USB_SD5				0x20	/*5 ms suspend            */
#define		USB_RESET_A 		0x40	/*reset  detected         */
#define		USB_RESUME_A		0x80	/*resume detected         */

/*--------- TXEV, TXMSK bits ---------*/
#define		USB_TXFIFO0			0x01	/*TX_DONE, FIFO 0         */
#define		USB_TXFIFO1			0x02	/*TX_DONE, FIFO 1         */
#define		USB_TXFIFO2			0x04	/*TX_DONE, FIFO 2         */
#define		USB_TXFIFO3			0x08	/*TX_DONE, FIFO 3         */
#define		USB_TXUDRN0			0x10	/*TX_URUN, FIFO 0         */
#define		USB_TXUDRN1			0x20	/*TX_URUN, FIFO 1         */
#define		USB_TXUDRN2			0x40	/*TX_URUN, FIFO 2         */
#define		USB_TXUDRN3			0x80	/*TX_URUN, FIFO 3         */


/*--------- RXEV, RXMSK bits ---------*/
#define		USB_RXFIFO0			0x01	/*RX_DONE, FIFO 0         */
#define		USB_RXFIFO1			0x02	/*RX_DONE, FIFO 1         */
#define		USB_RXFIFO2			0x04	/*RX_DONE, FIFO 2         */
#define		USB_RXFIFO3			0x08	/*RX_DONE, FIFO 3         */
#define		USB_RXOVRN0			0x10	/*RX_OVRN, FIFO 0         */
#define		USB_RXOVRN1			0x20	/*RX_OVRN, FIFO 1         */
#define		USB_RXOVRN2			0x40	/*RX_OVRN, FIFO 2         */
#define		USB_RXOVRN3			0x80	/*RX_OVRN, FIFO 3         */

/*------- NAKEV, NAKMSK bits ---------*/
#define		USB_NAK_I0			0x01	/*IN  NAK, FIFO 0         */
#define		USB_NAK_I1			0x02	/*IN  NAK, FIFO 1         */
#define		USB_NAK_I2			0x04	/*IN  NAK, FIFO 2         */
#define		USB_NAK_I3			0x08	/*IN  NAK, FIFO 3         */
#define		USB_NAK_O0			0x10	/*OUT NAK, FIFO 0         */
#define		USB_NAK_O1			0x20	/*OUT NAK, FIFO 1         */
#define		USB_NAK_O2			0x40	/*OUT NAK, FIFO 2         */
#define		USB_NAK_O3			0x80	/*OUT NAK, FIFO 3         */

/*--------- EPCX bits ----------------*/
#define		USB_EP_EN			0x10	/*enables endpt.  (1-6)   */
#define		USB_ISO				0x20	/*set for isochr. (1-6)   */
#define		USB_DEF				0x40	/*force def. adr (0 only) */
#define		USB_STALL			0x80	/*force stall handshakes  */

/*---------- TXCx bits ---------------*/
#define		USB_TX_EN			0x01	/*transmit enable         */
#define		USB_TX_LAST			0x02	/*last data in FIFO       */
#define		USB_TX_TOGL			0x04	/*specifies PID used      */
#define		USB_FLUSH			0x08	/*flushes all FIFO data   */
#define		USB_IGNIOS			0x80	/*       */

/*----------- TXSx bits --------------*/
#define		USB_TX_DONE			0x20	/*transmit done           */
#define		USB_ACK_STAT		0x40	/*ack status of xmission  */

/*------------ RXCx bits -------------*/
#define		USB_RX_EN			0x01	/*receive  enable         */
#define		USB_IGN_OUT			0x02	/*ignore out tokens       */
#define		USB_IGN_SETUP		0x04	/*ignore setup tokens     */

/*------------- RXS0 bits ------------*/
#define		USB_RX_LAST			0x10	/*indicates RCOUNT valid  */
#define		USB_RX_TOGL			0x20	/*last pkt was DATA1 PID  */
#define		USB_SETUP_RX		0x40	/*setup packet received   */
#define		USB_RX_ERR			0x80	/*last packet had an error*/
